#include <stdio.h>
#include <stdlib.h>
#include "msg/msg.h"
/* Create a log channel to have nice outputs. */
#include "xbt/log.h"
#include "xbt/asserts.h"

XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,"Messages specific for this msg example");

int master(int argc, char *argv[]);
int slave(int argc, char *argv[]);
MSG_error_t test_all(const char *platform_file, const char *application_file);

typedef enum {
  PORT_22 = 0,
  MAX_CHANNEL
} channel_t;

//keep a pointer to all surf running tasks.
#define NTASKS 2
int bool_printed=0;
double start_time, end_time, elapsed_time;
double   gl_data_size[NTASKS];
m_task_t gl_task_array[NTASKS];
int gl_task_array_id=0;

/** master */
int master(int argc, char *argv[])
{
  char *slavename = NULL;
  double task_comm_size = 0;
  m_task_t todo;
  m_host_t slave;
  //unique id to control statistics
  int id=gl_task_array_id++;

  /* data size */
  xbt_assert1(sscanf(argv[1],"%lg", &task_comm_size),
	      "Invalid argument %s\n", argv[1]);

  /* slave name */
  slavename = argv[2];
  
  { /*  Task creation.  */
    char sprintf_buffer[64] = "Task_0";
    todo = MSG_task_create(sprintf_buffer, 0, task_comm_size, NULL);
    //keep track of running tasks
    gl_task_array[id] = todo;
    gl_data_size[id]=task_comm_size;
  }

  { /* Process organisation */
    slave = MSG_get_host_by_name(slavename);
  }

  /* time measurement */
  double start_time = MSG_get_clock();
  MSG_task_put(todo, slave, PORT_22);
  double end_time = MSG_get_clock();
  INFO3("Send completed (to %s). Transfer time: %f\t Agregate bandwidth: %f",
	slave->name, (end_time - start_time), task_comm_size/(end_time-start_time));
  INFO2("Completed peer: %s time: %f", slave->name, (end_time-start_time));

  return 0;
} /* end_of_master */

/** Receiver function  */
int slave(int argc, char *argv[])
{
  m_task_t task = NULL;
  int a;
  int id=0;
  double remaining=0;

  a = MSG_task_get(&(task), PORT_22);
  if (a != MSG_OK) {
    INFO0("Hey?! What's up?");
    xbt_assert0(0,"Unexpected behavior.");
  }

  elapsed_time = MSG_get_clock() - start_time;

  if(!bool_printed){
    bool_printed=1;
    for(id=0; id<NTASKS; id++){
      if(gl_task_array[id] == NULL){
        INFO0("===> Task already done, skipping print statistics");
      }else if(gl_task_array[id] == task){
        INFO1("===> Bandwidth of first finishing (this) flow : %f.", gl_data_size[id]/elapsed_time);
      }else{
        remaining = MSG_task_get_remaining_communication(gl_task_array[id]);
        INFO1("===> Bandwidth of last finishing  flow        : %f.", (gl_data_size[id]-remaining)/elapsed_time);
      }
    }
  }

  MSG_task_destroy(task);

  return 0;
} /* end_of_slave */

/** Test function */
MSG_error_t test_all(const char *platform_file,
		     const char *application_file)
{
  MSG_error_t res = MSG_OK;

/*   MSG_config("workstation_model", "GTNETS"); */
  /* MSG_config("workstation_model","KCCFLN05"); */
  {				/*  Simulation setting */
    MSG_set_channel_number(MAX_CHANNEL);
    MSG_paje_output("msg_test.trace");
    MSG_create_environment(platform_file);
  }
  {                   /*   Application deployment */
    MSG_function_register("master", master);
    MSG_function_register("slave", slave);
    MSG_launch_application(application_file);
  }
  res = MSG_main();
  return res;
} /* end_of_test_all */


/** Main function */
int main(int argc, char *argv[])
{
  MSG_error_t res = MSG_OK;

  MSG_global_init(&argc,argv);
  if (argc < 3) {
     printf ("Usage: %s platform_file deployment_file\n",argv[0]);
     exit(1);
  }
  res = test_all(argv[1],argv[2]);

  MSG_clean();

  if(res==MSG_OK) return 0; 
  else return 1;
} /* end_of_main */















